#include <stdlib.h>
#include <stdio.h>
#include <locale.h>
#define dont_debug
#include "uwc.c"

void test_tomb (wchar_t c)
{
	char s [9];
	char us [9];
	int len = wctomb (s, c);
	int ulen = u_wctomb (us, c);
	int i;
	for (i = 0; i < len; i ++) {
		printf ("%02X ", (unsigned char) s [i]);
	}
	printf ("- ");
	for (i = 0; i < ulen; i ++) {
		printf ("%02X ", (unsigned char) us [i]);
	}
	printf ("\n");
}

void test_mbtowc_max (char * s, int max)
{
	wchar_t wc = 0x88888888;
	wchar_t uwc = 0x88888888;
	int len = mbtowc (& wc, s, max);
	int ulen = u_mbtowc (& uwc, s, max);
	printf ("<%s> (%02X) %d %04X u_ %d %04X\n", s, s ? (unsigned char) * s : 0xFFFF, len, wc, ulen, uwc);
}

void test_mbtowc (char * s)
{
	return test_mbtowc_max (s, 9);
}

void test_towc ()
{
	test_mbtowc_max ("", 1);
	test_mbtowc_max (" \r", 3);
	test_mbtowc_max (" \r", 4);

	test_mbtowc ("òôúùû");
	test_mbtowc ("òôúùû");

	test_mbtowc (0);
	test_mbtowc ("");
	test_mbtowc ("a");
	test_mbtowc ("ä");
	test_mbtowc ("€");
	test_mbtowc ("車");
	test_mbtowc_max ("車", 3);
	test_mbtowc_max ("車", 2);
	test_mbtowc_max ("車", 1);
	test_mbtowc_max ("車", 0);
/*
<#> (F0) -1 88888888 u_ 4 10300
<#> (81) 1 更 F901 u_ -1 88888888
*/
	test_mbtowc ("𐌀");
	test_mbtowc ("");
	test_mbtowc ("");
	test_mbtowc ("");
	test_mbtowc ("");
	test_mbtowc ("");
	test_mbtowc ("");

	test_mbtowc ("‧‧");
	test_mbtowc_max ("", 1);
	test_mbtowc ("‧‧");
#ifdef xx
	test_mbtowc_max ("", 1);
	test_mbtowc ("‧‧");
	test_mbtowc ("‧‧");
	test_mbtowc_max ("", 1);
	test_mbtowc ("abc");
	test_mbtowc_max ("", 1);
	test_mbtowc ("abc");
	test_mbtowc_max ("", 1);
	test_mbtowc ("abc");
#endif
}

void utflen (char * s)
{
	printf ("<%s> (%02X) %d u_ %d\n", s, (unsigned char) * s, mbtowc (0, s, 9), u_mbtowc (0, s, 9));
}

void test_utflen ()
{
	utflen ("a");
	utflen ("ä");
	utflen ("€");
	utflen ("車");
	utflen ("𑄑");
	utflen ("");
	utflen ("");
	utflen ("");
}

void test_mblen (char * s)
{
	printf ("<%s> mblen %d u_mblen %d\n", s, mblen (s, 9), u_mblen (s, 9));
}

void test_langinfo (int item)
{
	printf ("nl_langinfo (%d) %s u_ %s\n", item, nl_langinfo (item), u_nl_langinfo (item));
}

void test_mbstowcs (char * s)
{
	wchar_t wcs [9];
	wchar_t uwcs [9];
	int len = mbstowcs (wcs, s, 5);
	int len0 = wcslen (wcs);
	int ulen = u_mbstowcs (uwcs, s, 5);
	int ulen0 = u_wcslen (uwcs);
	int i;
	printf ("<%s> -> %d %d %d %d:", s, len, len0, ulen, ulen0);
	for (i = 0; i <= len; i ++) {
		printf (" %04X(%04X)", uwcs [i], wcs [i]);
	}
	printf ("\n");
}

void test_towcs ()
{
	test_mbstowcs ("aö€");
	test_mbstowcs ("a車x");
	test_mbstowcs ("aöbücß");
}

void test_wcstombs (char * s)
{
	wchar_t uwcs [9];
	char mbs [99];
	char umbs [99];
	int len;
	int ulen;
	(void) u_mbstowcs (uwcs, s, 9);
	len = wcstombs (mbs, uwcs, 88);
	ulen = u_wcstombs (umbs, uwcs, 88);
	printf ("<%s> -> (%d/%d) -> <%s>/<%s>\n", s, len, ulen, mbs, umbs);
}

void test_tombs ()
{
	test_wcstombs ("aä€");
	test_wcstombs ("a車x");
}

void test_wcwidth (char * s)
{
	wchar_t wc;
	int w;
	int uw;
	(void) u_mbtowc (& wc, s, 9);
	w = wcwidth (wc);
	uw = u_wcwidth (wc);
	printf ("width <%s> %d/%d\n", s, w, uw);
}

void test_wcswidth (char * s)
{
	wchar_t wcs [9];
	int w;
	int uw;
	(void) u_mbstowcs (wcs, s, 9);
	w = wcswidth (wcs, 9);
	uw = u_wcswidth (wcs, 9);
	printf ("width <%s> %d/%d\n", s, w, uw);
}

char * nulli (char * s)
{
	if (s) {
		return s;
	} else {
		return "(null)";
	}
}

int main ()
{
	printf ("%s %s\n", setlocale (LC_ALL, 0), setlocale (LC_CTYPE, 0));
	printf ("set ALL %s\n", nulli (setlocale (LC_ALL, "vi_VN")));
	printf ("%s %s\n", setlocale (LC_ALL, 0), setlocale (LC_CTYPE, 0));
	printf ("set CTYPE %s\n", nulli (setlocale (LC_CTYPE, "zh_CN.gb18030")));
	printf ("%s %s\n", setlocale (LC_ALL, 0), setlocale (LC_CTYPE, 0));
	printf ("set ALL %s\n", nulli (setlocale (LC_ALL, "en_US.UTF-8")));
	printf ("%s %s\n", setlocale (LC_ALL, 0), setlocale (LC_CTYPE, 0));

	printf ("set %s\n", setlocale (LC_ALL, ""));

	test_langinfo (CODESET);

	test_towc ();

#ifdef xx
	test_utflen ();
	test_mblen ("");
	test_mblen ("a");
	test_mblen ("ä");
	test_mblen ("€");

	test_tomb ('A');
	test_tomb ((unsigned char) '');
	test_tomb (0x20AC);

	test_towcs ();
	test_tombs ();

	test_wcwidth ("̀");
	test_wcwidth ("€");
	test_wcwidth ("車");
	test_wcswidth ("̀");
	test_wcswidth ("à");
	test_wcswidth ("a€");
	test_wcswidth ("a車");
	test_wcswidth ("a車€");
#endif
}
